# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_framework_qt.widgets import BaseWidget
from ftrack_qt.utils.decorators import invoke_in_qt_main_thread


class MayaChrSelectorWidget(BaseWidget):
    '''Text edit to set the collect object'''

    name = 'maya_chr_selector'
    ui_type = 'qt'

    def __init__(
        self,
        event_manager,
        client_id,
        context_id,
        plugin_config,
        group_config,
        on_set_plugin_option,
        on_run_ui_hook,
        parent=None,
    ):
        self._chr_le = None

        super(MayaChrSelectorWidget, self).__init__(
            event_manager,
            client_id,
            context_id,
            plugin_config,
            group_config,
            on_set_plugin_option,
            on_run_ui_hook,
            parent,
        )

    def pre_build_ui(self):
        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setAlignment(QtCore.Qt.AlignTop)
        self.setLayout(layout)
        self.setSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed
        )

    def build_ui(self):
        '''build function widgets.'''
        
        self._chr_le = QtWidgets.QLineEdit("No chr provided")
        can_override = self.plugin_config['options'].get(
            'can_override', False
        )
        self._chr_le.setReadOnly(not can_override)
        self.layout().addWidget(self._chr_le)

    def post_build_ui(self):
        '''hook events'''
        self._chr_le.textChanged.connect(self._on_chr_changed)

    def populate(self):
        '''Fetch info from plugin to populate the widget'''
        self.query_chr_object()

    def query_chr_object(self):
        '''
        Ask the plugin to process a query of first object that matches the expression, 
        we have to pass the expression to the plugin.
        '''
        payload = {
            'expression': self.plugin_config['options'].get('expression', '_char'),
            'collected_chr_object': self.plugin_config['options'].get('collected_chr_object')
        }
        self.run_ui_hook(payload)

    @invoke_in_qt_main_thread
    def ui_hook_callback(self, ui_hook_result):
        '''Handle the result of the UI hook.'''
        super(MayaChrSelectorWidget, self).ui_hook_callback(ui_hook_result)
        if ui_hook_result:
            self._chr_le.setText(ui_hook_result)
        else:
            self._chr_le.setText("No chr provided")

    def _on_chr_changed(self, chr_name):
        '''Updates the collected_chr_object option with the provided *chr_name'''
        if not chr_name:
            return
        self.set_plugin_option('collected_chr_object', chr_name)
